# Generated by Django 3.0.3 on 2020-03-05 12:14

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0005_accounttype_parent'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='date_create',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]
