# Generated by Django 3.0.3 on 2020-03-07 15:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0009_transaction_edited'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='account',
            options={'ordering': ['name']},
        ),
    ]
