# Generated by Django 3.0.3 on 2020-03-03 20:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0002_auto_20200303_2358'),
    ]

    operations = [
        migrations.AlterField(
            model_name='account',
            name='acc_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dbaccounting.AccountType', verbose_name='account type'),
        ),
        migrations.AlterField(
            model_name='accounttype',
            name='bal_type',
            field=models.CharField(choices=[('C', 'Credit'), ('D', 'Debit')], help_text='Indicate whether the account type is a credi or debit', max_length=1, verbose_name='balance type'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='from_acc',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='from_account', to='dbaccounting.Account', verbose_name='from account'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='to_acc',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='to_account', to='dbaccounting.Account', verbose_name='to account'),
        ),
    ]
