# Generated by Django 3.0.3 on 2020-03-06 17:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0006_account_date_create'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accounttype',
            name='parent',
            field=models.ForeignKey(blank=True, help_text='Is this a subcategory (e.g. Current Assets)', null=True, on_delete=django.db.models.deletion.CASCADE, to='dbaccounting.AccountType'),
        ),
    ]
