# Generated by Django 3.0.3 on 2020-03-06 20:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0007_auto_20200306_2118'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='updating',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dbaccounting.Transaction'),
        ),
    ]
