# Generated by Django 3.0.3 on 2020-03-04 20:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbaccounting', '0003_auto_20200304_0000'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='accounttype',
            options={'ordering': ['-bal_type', 'name']},
        ),
        migrations.AlterModelOptions(
            name='transaction',
            options={'ordering': ['date']},
        ),
        migrations.AlterField(
            model_name='accounttype',
            name='bal_type',
            field=models.CharField(choices=[('C', 'Credit'), ('D', 'Debit')], help_text='Indicate whether the account type is a credit or debit', max_length=1, verbose_name='balance type'),
        ),
    ]
